package org.amse.gk.grapheditor.algorithms.impl;

import java.awt.Color;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;


import org.amse.gk.grapheditor.algorithms.*;
import org.amse.gk.grapheditor.model.*;

@SuppressWarnings("serial")
public class BreadthFirstSearch implements IAlgorithm {
	private IGraph myGraph;	
	private List<IVertex> myActiveVertices = new ArrayList<IVertex>();
	private List<IVertex> myVisitedVertices = new ArrayList<IVertex>();
	private List<IVertex> myDelayedVertices =  new ArrayList<IVertex>();
	
	private List<IEdge> myActiveEdges = new ArrayList<IEdge>();
	
	private Queue<IVertex> myQueue = new LinkedList<IVertex>(); 
	
	private IVertex myCurrentVertex;
	private IStep myFirstStep;
	
	public BreadthFirstSearch(IGraph graph) {		
		myGraph = graph;
		final ChooseStartVertexStep chooseStartVertex = new ChooseStartVertexStep();
		final ChooseNextVertexStep chooseNextVertex = new ChooseNextVertexStep(); 
		final SelectAdjacentVerticesStep selectAdjacentVertices = new SelectAdjacentVerticesStep();
		final AddVerticesToQueueStep addVerticesToQueue = new AddVerticesToQueueStep();
		
		chooseStartVertex.setNextStep(chooseNextVertex);
		chooseNextVertex.setNextStep(selectAdjacentVertices);
		selectAdjacentVertices.setNextStep(addVerticesToQueue);
		addVerticesToQueue.setNextStep(chooseNextVertex);
		
		myFirstStep = chooseStartVertex;
	}	
	
	public IStep getFirstStep() {
		return myFirstStep;
	}	
	

	public Color getColor(IElement e) {
		if (e instanceof IVertex) {
			if (myActiveVertices.contains(e)) { 
				return Color.RED;
			} else if (myDelayedVertices.contains(e)) {
				return Color.GRAY;
			} else if (myVisitedVertices.contains(e)) {
				return Color.BLACK;
			}			 
		} else {
			if (myActiveEdges.contains(e)) {
				return Color.GRAY;
			}
		}
		return Color.LIGHT_GRAY;
	}			
	
	public String getDescription() {
		return "Breadth-first search algorithm traverses a graph, using a queue.\n" +
				"First you should choose the start vertex, it will be the first vertex in our queue.\n";
	}	
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("Active vertices: \n");
		for (IVertex v : myActiveVertices) {
			sb.append(v.getName());
			sb.append("\n");
		}
		sb.append("Active edges: \n");
		for (IEdge e : myActiveEdges) {
			sb.append(e.getVertexFrom().getName());
			sb.append(" ");
			sb.append(e.getVertexTo().getName());
			sb.append("\n");
		}
		sb.append("Delayed vertices: \n");
		for (IVertex v : myDelayedVertices) {
			sb.append(v.getName());
			sb.append("\n");
		}
		sb.append("Visited vertices: \n");
		for (IVertex v : myVisitedVertices) {
			sb.append(v.getName());
			sb.append("\n");
		}
		return sb.toString();
	}
	
	class ChooseNextVertexStep implements IStep {
		IStep myNextStep;	
		
		public void run() {
			myCurrentVertex = myQueue.poll();
			myDelayedVertices.remove(myCurrentVertex);	
			myActiveVertices.add(myCurrentVertex);					
		}
		
		public IStep nextStep() {			
			return myNextStep;
		}
		
		public void setNextStep (IStep nextStep) {
			myNextStep = nextStep;
		}
		
		public StepType getStepType() {		
			return !myQueue.isEmpty() || !myActiveVertices.isEmpty() ? StepType.NEXT_BUTTON_STEP : StepType.FINAL_STEP;
		}
		
		public String getAfterStepDescription() {			
			return "1. We take the first vertex from our queue " +
			"(vertices with gray color).\nIt becomes red.\n";
		}

		public String getBeforeStepDescription() {			
			if (getStepType() == StepType.FINAL_STEP)
				return "Now the queue is empty and all accessible vertices are visited (became black).\n" +
						"Algorithm is finished.";
			return "";
		}
	}
	
	class SelectAdjacentVerticesStep implements IStep {
		IStep myNextStep;		
		
		public void run() {
			myActiveVertices.clear();
			myVisitedVertices.add(myCurrentVertex);
			for (IEdge edge : myGraph.edges()) {					
				if (edge.getVertexFrom() == myCurrentVertex) {
					myActiveEdges.add(edge);
					IVertex to = edge.getVertexTo();
					if (!myVisitedVertices.contains(to) && !myQueue.contains(to)) {
						myActiveVertices.add(to);							
					}
				}
			}			
		}
		
		public IStep nextStep() {			
			return myNextStep;
		}
		
		public void setNextStep (IStep nextStep) {
			myNextStep = nextStep;
		}
		
		public StepType getStepType() {
			return StepType.NEXT_BUTTON_STEP;
		}
		
		public String getAfterStepDescription() {			
			return "2. Now we should find all vertices " +
			"adjacent to active.\nThe edges that lead us to such vertices becomes black.\n";
		}

		public String getBeforeStepDescription() {
			return "";
		}
	}

	class AddVerticesToQueueStep implements IStep {
		IStep myNextStep;	
		
		public void run() {
			myDelayedVertices.addAll(myActiveVertices);
			myQueue.addAll(myActiveVertices);
			myActiveVertices.clear();
			myActiveEdges.clear();
		}
		
		public IStep nextStep() {			
			return myNextStep;
		}
		
		public void setNextStep (IStep nextStep) {
			myNextStep = nextStep;
		}
		
		public StepType getStepType() {
			return StepType.NEXT_BUTTON_STEP;
		}
		
		public String getAfterStepDescription() {			
			return "3. All vertices are incident to " +
			"active vertex we added to queue.\nThey becomes gray.\n" +
			"So we should repeat this procedure, while our queue isn't empty. \n" +
			"..........\n";
		}

		public String getBeforeStepDescription() {			
			return "";
		}
	}
	
	class ChooseStartVertexStep implements IChooseVertexStep {
		IStep myNextStep;	

		public String getAfterStepDescription() {				
			return "";
		}

		public StepType getStepType() {				
			return StepType.CHOOSING_VERTEX_STEP;
		}

		public IStep nextStep() {				
			return myNextStep;
		}

		public void run() {}

		public void setNextStep(IStep step) {
			myNextStep = step;
		}

		public void setVertex(IVertex v) {			
			myActiveEdges.clear();
			myActiveVertices.clear();
			myDelayedVertices.clear();
			myVisitedVertices.clear();
			myDelayedVertices.add(v);
			myCurrentVertex = v;	
			myQueue.add(v);
		}

		public String getBeforeStepDescription() {			
			return "Please, choose a start vertex...\n";
		}
	}
	
}
