package org.amse.gk.grapheditor.gui.view;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;

import org.amse.gk.grapheditor.GraphEditor;
import org.amse.gk.grapheditor.algorithms.creators.BreadthFirstSearchCreator;
import org.amse.gk.grapheditor.gui.*;
import org.amse.gk.grapheditor.gui.view.states.AddEdgeState;
import org.amse.gk.grapheditor.gui.view.states.AddVertexState;
import org.amse.gk.grapheditor.gui.view.states.AlgorithmState;
import org.amse.gk.grapheditor.gui.view.states.DeleteState;
import org.amse.gk.grapheditor.gui.view.states.SelectState;
import org.amse.gk.grapheditor.io.*;
import org.amse.gk.grapheditor.model.IGraph;
import org.amse.gk.grapheditor.model.impl.GraphFactory;

@SuppressWarnings("serial")
public class View extends JPanel {
	
	private MainFrame myMainFrame;
	private GraphicPanel myGraphicPanel;
	
	private MySplitPane mySplitPane;
	private JPanel myTextPanel;		

	private String myFileName = null;
	private String myDirectory = null;	
	
	private IViewState myActiveState;
	
	private List<Action> myStates = new ArrayList<Action>();
	private List<Action> myAlgorithms = new ArrayList<Action>();
	private Action mySaveAction;
	private Action myNewAction;
	private Action myOpenAction;
	
	public View(IGraph graph, MainFrame mainFrame) {
		super(new BorderLayout());
		myTextPanel = new JPanel();
		myTextPanel.setLayout(new BorderLayout());	
		myTextPanel.setVisible(false);
		
		myGraphicPanel = new GraphicPanel(graph, this);
		myMainFrame = mainFrame;
		
		mySplitPane = new MySplitPane(MySplitPane.VERTICAL_SPLIT, true);		
		mySplitPane.setLeftComponent(myGraphicPanel);		
		mySplitPane.setRightComponent(myTextPanel);
		mySplitPane.hideTextComponent();
		
		add(mySplitPane, BorderLayout.CENTER);		

		IViewState defaultState = new AddVertexState(myGraphicPanel, "AddVertex.gif");
		myStates.add(defaultState);		
		myStates.add(new AddEdgeState(myGraphicPanel, "AddEdge.gif"));
		myStates.add(new SelectState(myGraphicPanel, "Select.gif"));
		myStates.add(new DeleteState(myGraphicPanel, "Delete.gif"));
		setActiveState(defaultState);		
		
		myAlgorithms.add(new AlgorithmState(new BreadthFirstSearchCreator(), this));
		
		mySaveAction = this.new SaveAction();
		myNewAction = this.new NewAction();
		myOpenAction = this.new OpenAction();
			
		myGraphicPanel.setSize(getMaximumSize());
		myGraphicPanel.updateAll();
		
		setVisible(true);
	}
	
	public GraphicPanel getGraphicPanel() {
		return myGraphicPanel;
	}
	
	public JPanel getTextPanel() {
		return myTextPanel;
	}
	
	public void hideTextComponent() {
		mySplitPane.hideTextComponent();
	}
	
	public void showTextComponent() {
		mySplitPane.showTextComponent();
	}
	
	public IViewState getActiveState() {
		return myActiveState;
	}
	
	public void setActiveState(Action state) {
		myActiveState = (IViewState)state;
		for (MouseListener ml : myGraphicPanel.getMouseListeners()) {
			myGraphicPanel.removeMouseListener(ml);
		}		
		
		for (MouseMotionListener mml : myGraphicPanel.getMouseMotionListeners()) {
			myGraphicPanel.removeMouseMotionListener(mml);
		}

		myGraphicPanel.addMouseListener(myActiveState);
		myGraphicPanel.addMouseMotionListener(myActiveState);
	}
	
	public List<Action> states() {
		return Collections.unmodifiableList(myStates);
	}
	
	public List<Action> algorithms() {
		return Collections.unmodifiableList(myAlgorithms);
	}
	
	public void setDirectory(String directory) {
		myDirectory = directory;
	}
	
	private void saveLastGraph() {
		GraphWriter w  = new GraphWriter(getGraphicPanel().getGraph());
		w.writeGraph(new File(System.getProperty("user.home") +
				"/.graph.grf"));			
			
	}
	
	private void saveWindowSettings() {
		String state = getActiveState().getValue(Action.NAME).toString();		
		new Settings(myMainFrame.getX(), myMainFrame.getY(), 
				myMainFrame.getWidth(), myMainFrame.getHeight(), state, myDirectory).save();	
	}
	
	public void updateAll() {
		for (Action action : myStates) {
			action.setEnabled(action.isEnabled());
		}
		for (Action action : myAlgorithms) {
			action.setEnabled(action.isEnabled());
		}
		mySaveAction.setEnabled(mySaveAction.isEnabled());		
		myNewAction.setEnabled(myNewAction.isEnabled());
		myOpenAction.setEnabled(myOpenAction.isEnabled());
	}	
	
	public class NewAction extends AbstractAction {

		public NewAction() {
			putValue(NAME, "New");
		}
		
		public void actionPerformed(ActionEvent e) {	
			if (getGraphicPanel().isModified()) {
				if (!saveGraphConfirmDialog()) {
					return ;
				}
			}
			getGraphicPanel().setGraph(GraphFactory.createGraph());
			getGraphicPanel().updateAll();		
			getGraphicPanel().setModified(false);
			myFileName = null;
		}
		
		public boolean isEnabled() {
			return true;
		}
		
	}	
	
	public class ApplyDefaultLayoutAction extends AbstractAction {

		private static final long serialVersionUID = 1L;
		
		public ApplyDefaultLayoutAction() {			
			URL u = GraphEditor.class.getResource("utils/icons/defaultlayout.gif");
			if (u != null) {
				putValue(SMALL_ICON, new ImageIcon(u));
			}
			putValue(AbstractAction.SHORT_DESCRIPTION, "Apply default layout (vertices will be placed by circle)");
		}

		public void actionPerformed(ActionEvent e) {
			GraphicPanel graphicPanel = getGraphicPanel();
			graphicPanel.getGraph().applyDefaultLayout(graphicPanel.getWidth()/2, 
					graphicPanel.getHeight()/2, Math.min(graphicPanel.getWidth(), graphicPanel.getHeight())* 3 / 10 );
			graphicPanel.repaint();
		}
		
	}
	
	public class OpenAction extends AbstractAction {

		private static final long serialVersionUID = 1L;
		
		public OpenAction() {
			putValue(AbstractAction.NAME, "Open...");
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke('O', KeyEvent.CTRL_MASK));
		}

		public void actionPerformed(ActionEvent e) {
			if (getGraphicPanel().isModified()) {
				if (!saveGraphConfirmDialog()) {
					return ;
				}
			}
			getGraphicPanel().setModified(false);
			JFileChooser chooser = new JFileChooser();			
			chooser.setVisible(true);	
			chooser.setFileFilter(new MyFileFilter());
			if (myDirectory != null) {
				chooser.setCurrentDirectory(new File(myDirectory));
			}
			int result = chooser.showOpenDialog(myMainFrame);			
			
			if (result == JFileChooser.APPROVE_OPTION) {
				IGraph graph = GraphFactory.createGraph();
				GraphReader reader = new GraphReader(chooser.getSelectedFile());
				try {
					reader.readGraph(graph);
				} catch (ReadGraphException ex) {
					JOptionPane.showMessageDialog(null, ex.getMessage(), 
							"Error", JOptionPane.ERROR_MESSAGE);
				}
				getGraphicPanel().setGraph(graph);			
			}			
			updateAll();
		}	
		
		public boolean isEnabled() {
			return true;
		}
	}
	
	public class SaveAction extends AbstractAction {

		private static final long serialVersionUID = 1L;
		
		public SaveAction() {
			putValue(AbstractAction.NAME, "Save");
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke('S', KeyEvent.CTRL_MASK));
		}		
	
		public void actionPerformed(ActionEvent e) {
			if (getGraphicPanel().isModified()) {
				if (myFileName != null) {
					File f = new File(myFileName);
					GraphWriter writer = new GraphWriter(getGraphicPanel().getGraph());
					writer.writeGraph(f);
				} else {
					if (saveGraph()) {
						getGraphicPanel().setModified(false);
					}
				}			
				updateAll();	
			}
		}		
		
		@Override 
		public boolean isEnabled() {
			return getGraphicPanel().isModified();
		}		
	}	
	
	public Action getSaveAction() {
		return mySaveAction;
	}
	
	public Action getNewAction() {
		return myNewAction;
	}
	
	public Action getOpenAction() {
		return myOpenAction;
	}

	public class SaveAsAction extends AbstractAction {

		private static final long serialVersionUID = 1L;
		
		public SaveAsAction() {
			putValue(AbstractAction.NAME, "Save As...");
		}		
	
		public void actionPerformed(ActionEvent e) {
			
			if (saveGraph()) {
				getGraphicPanel().setModified(false);
			}
			updateAll();			
		}		
				
	}	
	
	public class ExitAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public ExitAction() {
			putValue(NAME, "Exit");
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke('X', KeyEvent.ALT_MASK));			
		}		
		
		public void actionPerformed(ActionEvent e) {			
			saveLastGraph();
			saveWindowSettings();
			System.exit(0);					
		}
	}	
	
	@SuppressWarnings("serial")
	public class AboutAction extends AbstractAction {
			
		public AboutAction() {
			putValue(NAME, "About");
		}

		public void actionPerformed(ActionEvent e) {	
			JLabel l1 = new JLabel("GraphEditor v0.1");
			l1.setAlignmentX(JLabel.CENTER_ALIGNMENT);
			JLabel l2 = new JLabel("by Gulordava Christina");
			l2.setAlignmentX(JLabel.CENTER_ALIGNMENT);
			JLabel l3 = new JLabel("AMSE", JLabel.HORIZONTAL);
			l3.setAlignmentX(JLabel.CENTER_ALIGNMENT);
			JPanel p = new JPanel();
			BoxLayout bl = new BoxLayout(p, BoxLayout.Y_AXIS);
			p.setLayout(bl);
			p.add(l1);
			p.add(l2);
			p.add(l3);
			JOptionPane.showMessageDialog(null, p, "About", JOptionPane.INFORMATION_MESSAGE);
		}

	}
	
	private boolean saveGraphConfirmDialog() {
		Integer applyChanges = JOptionPane.showConfirmDialog(null, 
				"Graph has been modified. Save changes?", 
			    "Save Dialog", JOptionPane.YES_NO_CANCEL_OPTION);
		if (applyChanges == JOptionPane.YES_OPTION) {
			saveGraph();
		} else if (applyChanges == JOptionPane.CANCEL_OPTION) {
			return false;
		}
		return true;
		
	}
	
	private boolean saveGraph() {
		JFileChooser chooser = new JFileChooser();			
		chooser.setVisible(true);
		chooser.setFileFilter(new MyFileFilter());
		if (myDirectory != null) {
			chooser.setCurrentDirectory(new File(myDirectory));
		}
		if (myFileName != null) {
			chooser.setSelectedFile(new File(myFileName));
		}
		int result = chooser.showSaveDialog(myMainFrame);				
		if (result == JFileChooser.APPROVE_OPTION) {
			File f = null;
			try {
				myFileName = chooser.getSelectedFile().getCanonicalPath();
				if (!myFileName.endsWith(".grf")) {
					myFileName = myFileName + ".grf";
				}
				f = new File(myFileName);
				myDirectory = chooser.getCurrentDirectory().getCanonicalPath();	
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null, "Can't write to this file", 
						"Bad output file", JOptionPane.ERROR_MESSAGE);
			}				
			GraphWriter writer = new GraphWriter(getGraphicPanel().getGraph());
			writer.writeGraph(f);	
			return true;
		}
		return false;
		
		
	}		
	
	
	static class MyFileFilter extends FileFilter {

		@Override
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			return (f.getName().endsWith(".grf"));				
		}

		@Override
		public String getDescription() {					
			return "Graph files (*.grf)";
		}		
	}

}


