package org.amse.gk.grapheditor.gui.view.states;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import org.amse.gk.grapheditor.algorithms.*;
import org.amse.gk.grapheditor.gui.view.*;
import org.amse.gk.grapheditor.model.*;

@SuppressWarnings("serial")
public class AlgorithmState extends StateAdaptor {
	
	private IAlgorithmCreator myAlgorithmCreator;
	private IAlgorithm myAlgorithm;
	private IStep myCurrentStep;
	
	private GraphicPanel myGraphicPanel;
	private View myView;
	
	private Action myPrevActiveState;
	
	private JButton myNextButton;
	private JButton myCancelButton;	
	private JPanel myButtonPanel; 
	
	private JScrollPane myPane;
	private JTextArea myTextArea;

	public AlgorithmState(IAlgorithmCreator creator, View view) {
		super(view.getGraphicPanel(), "Algorithm", null);
		myGraphicPanel = view.getGraphicPanel();
		myView = view;
		myAlgorithmCreator = creator;
		
		// Create a button to do next step 
		myNextButton = new JButton(new NextStepAction());
		
		// Create a cancel button
		myCancelButton = new JButton(new CancelAlgorithmAction());		
		
		// Create a button panel
		myButtonPanel = new JPanel();
		myButtonPanel.setLayout(new FlowLayout());
		myButtonPanel.add(myNextButton);
		myButtonPanel.add(myCancelButton); 
		
		// Create a text area to display explanation tips 
		myTextArea = new JTextArea();
		myTextArea.setEditable(false);		
		myTextArea.setFont(new Font("Verdana", 0, 10));
		myTextArea.setLineWrap(false);
		
		myPane = new JScrollPane(myTextArea);	
		myPane.setAutoscrolls(true);		
		
		myView.getTextPanel().add(myPane);
		myView.getTextPanel().add(myButtonPanel, BorderLayout.SOUTH);		
		
		putValue(NAME, myAlgorithmCreator.getName());			
	}	

	public void actionPerformed(ActionEvent e) {	
		myAlgorithm = myAlgorithmCreator.newAlgorithm(myGraphicPanel.getGraph());
		myCurrentStep = myAlgorithm.getFirstStep();
		 
		myPrevActiveState = myView.getActiveState();
		myView.setActiveState(this);
		for (Action a : myView.states()) {
			a.setEnabled(false);
		}
		for (Action a : myView.algorithms()) {
			a.setEnabled(false);
		}
		myView.getNewAction().setEnabled(false);
		myView.getOpenAction().setEnabled(false);
		
		myNextButton.setText("Next>>");
		myNextButton.setVisible(true);	
		
		myView.showTextComponent();			
		myView.validate();		
		myView.repaint();
		
		myTextArea.setText(myAlgorithm.getDescription() + myCurrentStep.getBeforeStepDescription());
		setStep(myCurrentStep);
	}
	
	
	public String getDescription() {		
		return "Run algorithm";
	}
	
	private void setStep(IStep step) {
		myCurrentStep = step;
		
		switch (myCurrentStep.getStepType()) {
		case CHOOSING_VERTEX_STEP:
			myNextButton.setEnabled(false);
			break;
		case NEXT_BUTTON_STEP:
			myNextButton.setEnabled(true);			
			break;
		case FINAL_STEP:			
			myView.setActiveState(myPrevActiveState);
			myView.hideTextComponent();
			myView.validate();				
			myView.repaint();
			boolean isModified = myGraphicPanel.isModified();
			myGraphicPanel.updateAll();		
			myGraphicPanel.setModified(isModified);
			break;
		}				
	}
	
	private void nextStep() {		
		myCurrentStep.run();		
		myTextArea.setText(myTextArea.getText() + myCurrentStep.getAfterStepDescription());		
		myGraphicPanel.repaint();	
		
		setStep(myCurrentStep.nextStep());	
		
		if (myCurrentStep.nextStep().getStepType() == StepType.FINAL_STEP) {
			myNextButton.setText("Finish");
			myTextArea.setText(myTextArea.getText() + myCurrentStep.nextStep().getBeforeStepDescription());			
		} else {
			myTextArea.setText(myTextArea.getText() + "Please click \"Next\" to continue.\n");			
		}
		
	}
	
	class NextStepAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public NextStepAction() {
			putValue(NAME, "Next>>");
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke('X', KeyEvent.ALT_MASK));			
		}		
		
		public void actionPerformed(ActionEvent e) {
			nextStep();		
						
		}
	}
	
	class CancelAlgorithmAction extends AbstractAction {
	
		public CancelAlgorithmAction() {
			putValue(AbstractAction.NAME, "Cancel");
		}	

		public void actionPerformed(ActionEvent e) {			
			myView.setActiveState(myPrevActiveState);							
			myView.hideTextComponent();			
			myView.validate();							
			myView.repaint();
			myGraphicPanel.updateAll();	
			boolean isModified = myGraphicPanel.isModified();
			myGraphicPanel.updateAll();		
			myGraphicPanel.setModified(isModified);			
		}
		
	}

	public Color getColor(IElement e) {		
		if (myCurrentStep.getStepType() == StepType.CHOOSING_VERTEX_STEP) {
			return super.getColor(e);
		} else {
			return myAlgorithm.getColor(e);
		}
	}

	public boolean isEnabled() {
		return myAlgorithmCreator.isAlgorithmEnabled(myGraphicPanel.getGraph());
		
	}

	@Override
	public void mousePressed(MouseEvent e) {
		if (myCurrentStep.getStepType() == StepType.CHOOSING_VERTEX_STEP) {
			IVertex v;			
			if ((v = getGraphicPanel().getVertex(e.getX(), e.getY())) != null){
				unselectAllElements();			
				if (myCurrentStep instanceof IChooseVertexStep) {
					IChooseVertexStep step = (IChooseVertexStep)myCurrentStep;
					step.setVertex(v);					
				}
				nextStep();
			}		
		}		
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		if (myCurrentStep.getStepType() == StepType.CHOOSING_VERTEX_STEP) {
			IVertex vertex;			
			unselectAllElements();		
			if ((vertex = getGraphicPanel().getVertex(e.getX(), e.getY())) != null){
					selectElement(vertex);				
			} 
			getGraphicPanel().repaint();		
		}		
	}
	
	
}
