package org.amse.gk.grapheditor.model.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.amse.gk.grapheditor.model.IEdge;
import org.amse.gk.grapheditor.model.IGraph;
import org.amse.gk.grapheditor.model.IVertex;

/**
 * Class AbstractGraph is implementation for {@link IGraph}, provided all
 * basic operations. It contains a graph structure using a list of vertices 
 * {@link IVertex} and a list of directed edges {@link IEdge}. 
 * 
 * @see IGraph
 * @see IVertex
 * @see IEdge 
 */

class AbstractGraph implements IGraph {
	/**
	 * List of vertices 
	 */
	private List<IVertex> myVertexList = new ArrayList<IVertex>();
	
	/**
	 * List of directed edges 
	 */
	private List<IEdge> myEdgeList = new ArrayList<IEdge>();

	/**
	 * Vertex counter used for vertex autonumeration
	 */
	private int myVertexCounter = 0;
	
	/**
	 * Default constructor. Constructs an empty graph.
	 *
	 */
	public AbstractGraph() {
	}

	/**
	 * @return an unmodifiable list of graph vertices 
	 */
	public List<IVertex> vertices() {		
		return Collections.unmodifiableList(myVertexList);		
	}
	
	/**
	 * @return an unmodifiable list of graph edges
	 */
	public List<IEdge> edges() {		
		return Collections.unmodifiableList(myEdgeList);
	}
	
	/**
	 * Function creates and adds new vertex to the graph and automatically names it
	 * using vertex counter.
	 * 
	 * @param x   X-coordinate of the new vertex
	 * @param y   Y-coordinate of the new vertex
	 * 
	 * @return reference to the new vertex
	 */
	public IVertex addVertex(int x, int y) {
		IVertex v = new AbstractVertex("Vertex" + myVertexCounter++, x, y);
		myVertexList.add(v);
		return v;
	}
	
	/**
	 * Function creates and adds new edge to the graph if it is possible.
	 * 
	 * @param from   Starting vertex for the new edge
	 * @param to     Terminal vertex for the new edge
	 * 
	 * @return reference to the new edge, if it was added or null if it isn't
	 *         possible
	 */
	public IEdge addEdge(IVertex from, IVertex to) {
		if (!canAddEdge(from, to)) {
			return null;
		}
		IEdge e = new AbstractEdge(from, to);
		myEdgeList.add(e);
		return e;
	}
	
	/**
	 * Function verifies if it is possible to add a new edge with given
	 * starting and terminal vertices. We can't add new edge if there is
	 * already such edge of one of the parameters is null or a starting vertex
	 * is equal to terminal.
	 * 
	 * @param from   Starting vertex for the new edge
	 * @param to     Terminal vertex for the new edge
	 * 
	 * @return   true if it is possible or false otherwise
	 */
	public boolean canAddEdge(IVertex from, IVertex to) {
		if ((from == null) || (to == null) || (from == to)) {
			return false;
		}
		for (IEdge e : myEdgeList) {
			if (e.getVertexFrom() == from && e.getVertexTo() == to)
				return false;			
		}
		return true;
	}
	
	/**
	 * Removes a given vertex with all incident edges from the graph
	 * 
	 * @param   vertex to remove
	 */
	public void removeVertex(IVertex v) {
		for (Iterator<IEdge> it = myEdgeList.iterator(); it.hasNext();) {
			IEdge e = it.next();
			if ((e.getVertexFrom() == v) || (e.getVertexTo() == v)) {
				it.remove();
			}
		}
		myVertexList.remove(v);
	}
	
	/**
	 * Removes a given edge from the graph
	 * 
	 * @param   edge to remove
	 */
	public void removeEdge(IEdge e) {
		myEdgeList.remove(e);		
	}
	
	/**
	 * Function applies default layout to the graph. Default layout 
	 * places vertices by circle, according to their order.
	 * 
	 * @param cx   X-coordinate of the layout circle
	 * @param cy   Y-coordinate of the layout circle
	 * @param r    Circle radius 
	 */
	public void applyDefaultLayout(int cx, int cy, int r) {
		int n = myVertexList.size();
		
		if (n == 0) {
			return ;
		} else if (n == 1) {
			myVertexList.get(0).move(cx, cy);
			return ;
		}
		
		double pi = Math.PI;
		double alpha;
		int x, y;		
		int i = 0;
		for (IVertex v : myVertexList){
			alpha = 2 * pi * i / n;
			x = cx + (int)(r * Math.sin(alpha));
			y = cy - (int)(r * Math.cos(alpha));
			v.move(x, y);
			i++;
		}
		
	}
	
	@Override
	public String toString(){		
		StringBuffer sb = new StringBuffer();
		sb.append(myVertexList.size());		

		for (IVertex v : myVertexList) {
			sb.append('\n');		
			sb.append(v);
			sb.append(' ');
			sb.append(v.getX());
			sb.append(' ');
			sb.append(v.getY());		
		}		
		for (IEdge edge : edges()) {
			sb.append("\nEdge: ");
			sb.append(edge.getVertexFrom());
			sb.append(" -> ");
			sb.append(edge.getVertexTo());
		}
		return sb.toString();
	}
}
